/*
 * main.cpp
 *
 * This program shows how to use GSBMPLib with Borland C++ Builder 5.
 * Don`t forget to copy gs.dll and gsbmp.dll into the
 * working directory before runing!
 *
 * (C) ENTROPYWARE, Alexander Simakov.
 *
 * Our site: http://www.entropyware.info/
 * Support:  support@entropyware.info
 * Author:   xander@entropyware.info
 */

#include <vcl.h>
#pragma hdrstop

#include "main.h"

// Return codes for GSBMPLib

#define GSBMP_OK              (0) /* work is completed */
#define GSBMP_OPEN_ERROR     (-1) /* can`t open file */
#define GSBMP_CREATE_ERROR   (-2) /* can`t create file */
#define GSBMP_MEMORY_ERROR   (-3) /* not enough memory */
#define GSBMP_INTERNAL_ERROR (-4) /* internal library error */
#define GSBMP_DAMAGED_BMP    (-5) /* unsupported or damaged BMP file */
#define GSBMP_DAMAGED_GS     (-6) /* damaged GS file */

// Compression & decompression functions prototypes
typedef int (CALLBACK *lp_gsc)(char *in_path, char *out_path, int quality, void (__stdcall *my_progress)(int percent_done));
typedef int (CALLBACK *lp_gsd)(char *in_path, char *out_path, void (__stdcall *my_progress)(int percent_done));

HINSTANCE hLib;
lp_gsc gsc;
lp_gsd gsd;

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmMain *frmMain;

//---------------------------------------------------------------------------
// User supplied progress reporting function
void __stdcall Progress(int percent_done)
{
  frmMain->ProgressBar->Position = percent_done;
}

//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::TrackBarChange(TObject *Sender)
{
lblQuality->Caption = IntToStr(TrackBar->Position);
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::btnExitClick(TObject *Sender)
{
Application->Terminate();
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::radDecompressClick(TObject *Sender)
{
TrackBar->Enabled = false;
lblBestSize->Enabled = false;
lblBestQuality->Enabled = false;
lblCompressionQuality->Enabled = false;
lblQuality->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::radCompressClick(TObject *Sender)
{
TrackBar->Enabled = true;
lblBestSize->Enabled = true;
lblBestQuality->Enabled = true;
lblCompressionQuality->Enabled = true;
lblQuality->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::btnOpenClick(TObject *Sender)
{
if (OpenDialog->Execute()) edtInput->Text = OpenDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::btnSaveClick(TObject *Sender)
{
if (SaveDialog->Execute()) edtOutput->Text = SaveDialog->FileName;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::btnGoClick(TObject *Sender)
{
  char in_path[1024], out_path[1024];
  HWND hWnd;
  int res;

  hWnd = (HWND)frmMain->WindowHandle;

  StrPCopy(in_path, edtInput->Text);
  StrPCopy(out_path, edtOutput->Text);


  if (radCompress->Checked == true)
    res = (*gsc)(in_path, out_path, TrackBar->Position, Progress); // Compress
  else
    res = (*gsd)(in_path, out_path, Progress);                     // Decompress


  switch (res) {
    case GSBMP_OK:
    MessageBox(hWnd, "Done!", "GSBMPLib-CB5", MB_ICONINFORMATION);
    break;

    case GSBMP_OPEN_ERROR:
    MessageBox(hWnd, "Can`t open file.", "GSBMPLib-CB5", MB_ICONERROR);
    break;

    case GSBMP_CREATE_ERROR:
    MessageBox(hWnd, "Can`t create file.", "GSBMPLib-CB5", MB_ICONERROR);
    break;

    case GSBMP_MEMORY_ERROR:
    MessageBox(hWnd, "Not enough memory.", "GSBMPLib-CB5", MB_ICONERROR);
    break;

    case GSBMP_INTERNAL_ERROR:
    MessageBox(hWnd, "Internal library error.", "GSBMPLib-CB5", MB_ICONERROR);
    break;

    case GSBMP_DAMAGED_BMP:
    MessageBox(hWnd, "Unsupported or damaged BMP file.", "GSBMPLib-CB5", MB_ICONERROR);
    break;

    case GSBMP_DAMAGED_GS:
    MessageBox(hWnd, "Damaged GS file header.", "GSBMPLib-CB5", MB_ICONERROR);
    break;
  }

  ProgressBar->Position = 0;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormCreate(TObject *Sender)
{
  hLib = LoadLibrary("gsbmp.dll");
  gsc = (lp_gsc)GetProcAddress(hLib, "gs_compress_bmp");
  gsd = (lp_gsd)GetProcAddress(hLib, "gs_decompress_bmp");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::FormDestroy(TObject *Sender)
{
  FreeLibrary(hLib);
}
//---------------------------------------------------------------------------

