/*
 * gsbmplib.h
 *
 * Dynamic Link Library for compressing BMP files using GSLib.
 *
 * (C) ENTROPYWARE, Alexander Simakov.
 *
 * Our site: http://www.entropyware.info/
 * Support:  support@entropyware.info
 * Author:   xander@entropyware.info
 */

#ifndef GSBMPLIB_H
#define GSBMPLIB_H

#define GS_API __stdcall

#ifdef __cplusplus
extern "C" {
#endif

/*
 * GSBMPLib provides file-to-file compression and decompression functions
 * for Windows BMP images (8-bit grayscale and 24-bit truecolor).
 * Note GSBMPLib is based on GSLib (gsbmp.dll uses gs.dll).
 */

#define BUFFER_SIZE (4096)

/* Return codes */

#define GSBMP_OK              (0) /* work is completed */
#define GSBMP_OPEN_ERROR     (-1) /* can`t open file */
#define GSBMP_CREATE_ERROR   (-2) /* can`t create file */
#define GSBMP_MEMORY_ERROR   (-3) /* not enough memory */
#define GSBMP_INTERNAL_ERROR (-4) /* internal library error */
#define GSBMP_DAMAGED_BMP    (-5) /* unsupported or damaged BMP file */
#define GSBMP_DAMAGED_GS     (-6) /* damaged GS file */

int GS_API gs_compress_bmp(char *in_path, char *out_path, int quality, void (GS_API *my_progress)(int percent_done));

/* gs_compress_bmp compresses BMP file.
 *
 * in_path - path to open BMP file
 * out_path - path to create GS file
 * quality - compression quality
 *
 * The quality must be between 0 and 100. Increasing quality
 * degradates compression ratio, decreasing quality favors size.
 * The quality of 65 works fine in most cases.
 *
 * my_progress - progress reporting function.To avoid progress
 * reporting pass NULL instead.
 *
 * gs_compress_bmp returns GSBMP_OK if the image was compressed.
 * (see other return codes above).
 */

int GS_API gs_decompress_bmp(char *in_path, char *out_path, void (GS_API *my_progress)(int percent_done));

/* gs_decompress_bmp decompresses BMP file.
 *
 * in_path - path to open GS file
 * out_path - path to create BMP file
 * 
 * my_progress - progress reporting function.To avoid progress
 * reporting pass NULL instead.
 *
 * gs_decompress_bmp returns GSBMP_OK if the image was decompressed.
 * (see other return codes above).
 */

#ifdef __cplusplus
}
#endif

#endif /* GSBMPLIB_H */
