/*
 * psnr.c
 *
 * This program measures distortion by the PSNR (peak signal to noise ratio).
 *
 * (C) ENTROPYWARE, Alexander Simakov.
 *
 * Our site: http://www.entropyware.info/
 * Support:  support@entropyware.info
 * Author:   xander@entropyware.info
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "pbm.h"

void get_psnr(char *name1, char *name2)
{
  FILE *f1, *f2;
  pbm_hdr h1, h2;
  int ch1, ch2;
  double psnr = 0;

  if (((f1 = fopen(name1, "rb")) == NULL) || ((f2 = fopen(name2, "rb")) == NULL)) {
    printf("Can`t open file(s).\n");
    exit(1);
  }

  if ((read_hdr(f1, &h1) != 0) || (read_hdr(f2, &h2) != 0)) {
    printf("Corrupted file(s) or usupported format(s).\n");
    exit(1);
  }

  if ((h1.type != PGM) || (h2.type != PGM)) {
    printf("Only PGM (P5) format is supported.\n");
    exit(1);
  }

  if ((h1.width != h2.width) || (h1.height != h2.height)) {
    printf("Images must have equal dimentions.\n");
    exit(1);
  }

  do {
    ch1 = getc(f1);
    ch2 = getc(f2);
    if (ch1 != EOF && ch2 != EOF) psnr += (ch1 - ch2) * (ch1 - ch2);
  } while (ch1 != EOF && ch2 != EOF);

  if (psnr != 0.0) {
    psnr = 10 * log10(255.0 * 255.0 * h1.width * h1.height / psnr);
    printf("PSNR == %5.3f dB\n", psnr);
  }
  else printf("No differences encountered\n");

  fclose(f1);
  fclose(f2);
}

int main(int argc, char *argv[])
{
  printf("\nPSNR utility, (C) ENTROPYWARE, Alexander Simakov.\n");

  if (argc != 3) {
    printf("\nUsage: <psnr.exe> <file1.pgm> <file2.pgm>\n");
    exit(1);
  }

  else get_psnr(argv[1], argv[2]);

  return 0;
}
