/*
 * wings.c
 *
 * This program shows how to use GSBMPLib with Visual C++ 6.
 * Don`t forget to copy gs.dll and gsbmp.dll into the
 * working directory before runing!
 *
 * (C) ENTROPYWARE, Alexander Simakov.
 *
 * Our site: http://www.entropyware.info/
 * Support:  support@entropyware.info
 * Author:   xander@entropyware.info
 */

#include <windows.h>
#include <commctrl.h>
#include <commdlg.h>
#include "resource.h"
#include "gsbmplib.h"

HWND hDlg;

// User supplied progress report function
void __stdcall Progress(int PercentDone)
{
	SendDlgItemMessage(hDlg, IDC_PROGRESS, PBM_SETPOS, PercentDone, 0);
}

BOOL CALLBACK MainDialogProc(HWND hwndDlg,
                             UINT uMsg,
                             WPARAM wParam,
                             LPARAM lParam)
{
	OPENFILENAME ofn;
	int wmId, wmEvent, val, res;
	char InPath[1024], OutPath[1024];
	char buf[4];

	hDlg = hwndDlg;

	switch (uMsg) {
		case WM_INITDIALOG:
		// Check Compression
		CheckRadioButton(hwndDlg, IDC_RAD_COMPRESS, IDC_RAD_DECOMPRESS, IDC_RAD_COMPRESS);
		// Init Slider
		SendDlgItemMessage(hwndDlg, IDC_SLIDER,TBM_SETRANGE, (WPARAM) TRUE, (LPARAM) MAKELONG(0, 100));
		SendDlgItemMessage(hwndDlg, IDC_SLIDER,TBM_SETPOS, (WPARAM) TRUE, (LPARAM) 65);
		// Init ProgressBar
		SendDlgItemMessage(hwndDlg, IDC_PROGRESS, PBM_SETRANGE, 0, MAKELPARAM(0, 100));
		SendDlgItemMessage(hwndDlg, IDC_PROGRESS, PBM_SETPOS, 0, 0);
		break;

		case WM_CLOSE:
		EndDialog(hwndDlg, 0);		
		break;

		case WM_HSCROLL:
		val = SendDlgItemMessage(hwndDlg, IDC_SLIDER, TBM_GETPOS, 0, 0);
		wsprintf(buf, "%d", val);
		SetDlgItemText(hwndDlg, IDC_LBL_QUALITY, buf);
		break;

		case WM_COMMAND:
		wmId    = LOWORD(wParam);
		wmEvent = HIWORD(wParam);

		switch (wmId) {
			case IDC_BTN_OPEN:
			InPath[0] = 0;
			ofn.lStructSize = sizeof(ofn);
			ofn.hwndOwner = hwndDlg;
			ofn.hInstance = NULL;
			ofn.lpstrFilter = "Windows BMP (*.bmp)\0*.bmp\0Grayscale GS (*.gs)\0*.gs\0All files (*.*)\0*.*\0\0";
			ofn.lpstrCustomFilter = NULL;
			ofn.nMaxCustFilter = 0;
			ofn.nFilterIndex = 1;
			ofn.lpstrFile = InPath;
			ofn.nMaxFile = sizeof(InPath);
			ofn.lpstrFileTitle = NULL;
			ofn.nMaxFileTitle = 0;
			ofn.lpstrInitialDir = NULL;
			ofn.lpstrTitle = NULL;
			ofn.Flags = OFN_ENABLESIZING | OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
			ofn.lpstrDefExt = NULL;
			ofn.lCustData = 0;
			ofn.lpfnHook = NULL;
			ofn.lpTemplateName = NULL;

			if (GetOpenFileName(&ofn) == TRUE) {
				SetDlgItemText(hwndDlg, IDC_EDT_INPUT, InPath);
			}

			break;

			case IDC_BTN_SAVE:
			OutPath[0] = 0;
			ofn.lStructSize = sizeof(ofn);
			ofn.hwndOwner = hwndDlg;
			ofn.hInstance = NULL;
			ofn.lpstrFilter = "Windows BMP (*.bmp)\0*.bmp\0Grayscale GS (*.gs)\0*.gs\0All files (*.*)\0*.*\0\0";
			ofn.lpstrCustomFilter = NULL;
			ofn.nMaxCustFilter = 0;
			ofn.nFilterIndex = 1;
			ofn.lpstrFile = OutPath;
			ofn.nMaxFile = sizeof(OutPath);
			ofn.lpstrFileTitle = NULL;
			ofn.nMaxFileTitle = 0;
			ofn.lpstrInitialDir = NULL;
			ofn.lpstrTitle = NULL;
			ofn.Flags = OFN_ENABLESIZING | OFN_EXPLORER | OFN_HIDEREADONLY;
			ofn.lpstrDefExt = "";
			ofn.lCustData = 0;
			ofn.lpfnHook = NULL;
			ofn.lpTemplateName = NULL;

			if (GetSaveFileName(&ofn) == TRUE) {
				SetDlgItemText(hwndDlg, IDC_EDT_OUTPUT, OutPath);
			}

			break;

			case IDC_BTN_GO:
			GetDlgItemText(hwndDlg, IDC_EDT_INPUT, InPath, sizeof(InPath));
			GetDlgItemText(hwndDlg, IDC_EDT_OUTPUT, OutPath, sizeof(OutPath));

			if (IsDlgButtonChecked(hwndDlg, IDC_RAD_COMPRESS) == BST_CHECKED) {
				val = SendDlgItemMessage(hwndDlg, IDC_SLIDER, TBM_GETPOS, 0, 0);
				res = gs_compress_bmp(InPath, OutPath, val, Progress); // Compress file
			} else {
				res = gs_decompress_bmp(InPath, OutPath, Progress);    // Decompress file
			}

			switch (res) {
				case GSBMP_OK:
				MessageBox(hwndDlg, "Done!", "WinGS", MB_ICONINFORMATION);
				break;

				case GSBMP_OPEN_ERROR:
				MessageBox(hwndDlg, "Can`t open file.", "WinGS", MB_ICONERROR);
				break;

				case GSBMP_CREATE_ERROR:
				MessageBox(hwndDlg, "Can`t create file.", "WinGS", MB_ICONERROR);
				break;

				case GSBMP_MEMORY_ERROR:
				MessageBox(hwndDlg, "Not enough memory.", "WinGS", MB_ICONERROR);
				break;

				case GSBMP_INTERNAL_ERROR:
				MessageBox(hwndDlg, "Internal library error.", "WinGS", MB_ICONERROR);
				break;

				case GSBMP_DAMAGED_BMP:
				MessageBox(hwndDlg, "Unsupported or damaged BMP file.", "WinGS", MB_ICONERROR);
				break;

				case GSBMP_DAMAGED_GS:
				MessageBox(hwndDlg, "Damaged GS file header.", "WinGS", MB_ICONERROR);
				break;
			}

			Progress(0);

			break;

			case IDC_RAD_COMPRESS:
			EnableWindow(GetDlgItem(hwndDlg, IDC_LBL_BEST_SIZE), TRUE);
			EnableWindow(GetDlgItem(hwndDlg, IDC_LBL_BEST_QUALITY), TRUE);
			EnableWindow(GetDlgItem(hwndDlg, IDC_LBL_COMPRESSION_QUALITY), TRUE);
			EnableWindow(GetDlgItem(hwndDlg, IDC_LBL_QUALITY), TRUE);
			EnableWindow(GetDlgItem(hwndDlg, IDC_SLIDER), TRUE);
			EnableWindow(GetDlgItem(hwndDlg, IDC_STATIC3), TRUE);
			break;

			case IDC_RAD_DECOMPRESS:
			EnableWindow(GetDlgItem(hwndDlg, IDC_LBL_BEST_SIZE), FALSE);
			EnableWindow(GetDlgItem(hwndDlg, IDC_LBL_BEST_QUALITY), FALSE);
			EnableWindow(GetDlgItem(hwndDlg, IDC_LBL_COMPRESSION_QUALITY), FALSE);
			EnableWindow(GetDlgItem(hwndDlg, IDC_LBL_QUALITY), FALSE);
			EnableWindow(GetDlgItem(hwndDlg, IDC_SLIDER), FALSE);
			EnableWindow(GetDlgItem(hwndDlg, IDC_STATIC3), FALSE);
			break;

			case IDC_BTN_EXIT:
			EndDialog(hwndDlg, 0);
			break;
		}

		default: return FALSE;
    }

	return TRUE;
}


int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{

	InitCommonControls();

	DialogBox(hInstance, (LPCTSTR)IDD_MAIN, NULL, MainDialogProc);

	return 0;
}


